/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui.filelist;

import gui.util.GuiUtil;
import gui.util.tablemodel.ObjectTableModel;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.nio.file.DirectoryStream;
import java.nio.file.DirectoryStream.Filter;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.FileTime;
import java.util.LinkedList;
import java.util.Vector;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableRowSorter;


public class FileList extends JScrollPane {
	private static final long serialVersionUID = 1L;
	
	private JTable fileList;
	private FileTableModel listModel;
	private Path dir;
	
	private JPopupMenu popupMenu;
	private LinkedList<Filter<Path>> filters;
	
	private static final Filter<Path> NO_DIRS_FILTER=new Filter<Path>() {
		
		@Override
		public boolean accept(Path f) {
			return !Files.isDirectory(f);
		}
	};
	
	public Path getDir() {
		return dir;
	}
	
	public FileList(boolean multiselect,String colName,String colSize,String colDate) {
		listModel=new FileTableModel(); 
		fileList=new JTable(listModel) {
			private static final long serialVersionUID = 1L;

			private FileListCellRenderer cr = new FileListCellRenderer(FileList.this);
			
			@Override
			public String getToolTipText(MouseEvent e) {
				if (fileList.getSelectedRow()>-1) {
					return listModel.getValueAt(fileList.getSelectedRow(),0).toString();
				} else {
					return null;
				}
			}
			
			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
				if (column==0)
					return cr;
				else
					return super.getCellRenderer(row, column);
			}
		};
		listModel.addColumn(colName);
		listModel.addColumn(colSize);
		listModel.addColumn(colDate);
		
		fileList.setShowGrid(false);
		
		
		filters=new LinkedList<Filter<Path>>();
		addFileFilter(NO_DIRS_FILTER);
		fileList.setOpaque(true);
		fileList.setBackground(Color.WHITE); 
		
		if (multiselect)
			fileList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		else
			fileList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		fileList.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		
		setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		
		popupMenu=new JPopupMenu();
		JMenuItem selectAll=new JMenuItem("Select All");
		JMenuItem invert=new JMenuItem("Invert Selection");
		JMenuItem selectNone=new JMenuItem("Select None");
		JMenuItem mRefresh=new JMenuItem("Refresh");
		
		selectAll.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				fileList.setRowSelectionInterval(0, listModel.getRowCount()-1);
			}
		});
		
		selectNone.addActionListener(new ActionListener() {
					
			@Override
			public void actionPerformed(ActionEvent e) {
				fileList.clearSelection();
			}
		});
		
		mRefresh.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					loadDir(dir);
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			}
		});
		
		invert.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				int[] invert=new int[listModel.getRowCount()-fileList.getSelectedRows().length];
				
				int k=0;
				for (int i=0;i<listModel.getRowCount();i++) {
					
					if (!fileList.isRowSelected(i)) {
						invert[k]=i;
						k++;
					}
				}
				fileList.clearSelection();
				
				if (invert.length>0) {
					int index0=invert[0];
					for (int i=1;i<invert.length;i++) {
						
						if (invert[i]!=invert[i-1]+1) {
							fileList.getSelectionModel().addSelectionInterval(index0, invert[i-1]);
							index0=invert[i];
						} 
					}
					fileList.getSelectionModel().addSelectionInterval(index0, invert[invert.length-1]);
				
				}
				
			}
		});
		popupMenu.add(selectAll);
		popupMenu.add(selectNone);
		popupMenu.add(invert);
		popupMenu.add(mRefresh);
		
		fileList.add(popupMenu);
		fileList.addMouseListener(new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger()) {
					openPopup(e);
				}
			}
			
			@Override
			public void mousePressed(MouseEvent e) {
				if (e.isPopupTrigger()) {
					openPopup(e);
				}
			}
			
			@Override
			public void mouseExited(MouseEvent e) {}
			
			@Override
			public void mouseEntered(MouseEvent e) {}
			
			@Override
			public void mouseClicked(MouseEvent e) {}
		});
		
		fileList.addKeyListener(new KeyListener() {
			
			@Override
			public void keyTyped(KeyEvent e) {}
			
			@Override
			public void keyReleased(KeyEvent e) {}
			
			@Override
			public void keyPressed(KeyEvent e) {
				if (e.getKeyChar()!=KeyEvent.CHAR_UNDEFINED) {
					for (int i=fileList.getSelectedRow()+1;i<fileList.getRowCount();i++) {
						
						
						if (Character.toUpperCase(fileList.getValueAt(i, 0).toString().charAt(0))==Character.toUpperCase(e.getKeyChar())) {
							fileList.setRowSelectionInterval(i, i);
							fileList.scrollRectToVisible(fileList.getCellRect(i, 0, false));
							return;
						}
					}
				}
			}
		});
		
		setViewportView(fileList);
		
		fileList.setFillsViewportHeight(true);
		
		TableRowSorter<FileTableModel> s = new TableRowSorter<FileTableModel>(listModel);
		fileList.setRowSorter(s);
		fileList.validate();
	}
	
	private void openPopup(MouseEvent e) {
		popupMenu.show(e.getComponent(), e.getX(), e.getY());
	}
	
	
	public void loadDir(Path dir) throws IOException {
		
		if (dir==null) return;
		
		listModel.setRowCount(0);
		
		this.dir=dir;
		if (Files.isReadable(dir)) {
			DirectoryStream<Path> stream=Files.newDirectoryStream(dir,new Filter<Path>() {
	
				@Override
				public boolean accept(Path entry) throws IOException {
					for (Filter<Path> f:filters) {
						if (!f.accept(entry)) return false;
					}
					return true;
				}
			});
			for (Path f:stream) {
				Vector<Object> vFiles=new Vector<Object>();
				vFiles.add(new FileName(f.getFileName().toString()));
				vFiles.add(new FileSize( Files.size(f))) ;
				vFiles.add(new DisplayFileTime(Files.getLastModifiedTime(f)));
				listModel.addRow(f,vFiles);
			}
		}
	}
	
	public Path[] getSelectedFiles() {
		
		int[] rows=fileList.getSelectedRows();
		
		if (rows.length==0) return null; 
		Path[] f=new Path[rows.length];
		for (int i=0;i<rows.length;i++) {
			
			f[i]=listModel.getValue( fileList.convertRowIndexToModel(rows[i]));
		}
		
		return f;
	}
	
	public void refresh() throws IOException {
		loadDir(dir);
	}
	
	public void addFileFilter(Filter<Path> f) {
		if (f==null) return;
		filters.add(f);
	}
	
	public void removeAllFilters() {
		filters.clear();
	}
	
	private class FileSize implements Comparable<FileSize> {

		private long fs;
		
		public FileSize(long fs) {
			this.fs=fs;
		}
		
		@Override
		public int compareTo(FileSize o) {
			long thisVal=fs;
			long anotherVal=o.fs;
			return (thisVal<anotherVal ? -1 : (thisVal==anotherVal ? 0 : 1));
		}
		
		@Override
		public String toString() {
			return GuiUtil.formatBytes(fs);
		}
	}
	

	private class FileTableModel extends ObjectTableModel<Path> {
		private static final long serialVersionUID = 1L;
		
		@Override
		public boolean isCellEditable(int row,int col) {
			return false;
		}
		
		@Override
		public Class<?> getColumnClass(int columnIndex) {
			if (columnIndex==0)
				return FileName.class;
			else if (columnIndex==1)
				return FileSize.class;
			else if (columnIndex==2)
				return FileTime.class;
			else
				return super.getColumnClass(columnIndex);
		}
	}
	
	public void setColumnWidth(int width,int column) {
		fileList.getColumnModel().getColumn(column).setPreferredWidth(width);
	}
	
	public int getColumnWidth(int column) {
		return fileList.getColumnModel().getColumn(column).getPreferredWidth();
	}
	
	public void addSelectionListener(ListSelectionListener l) {
		fileList.getSelectionModel().addListSelectionListener(l);
	}
	
	public void addDoubleClickActionListener(final ActionListener a) {
		fileList.addMouseListener(new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent arg0) {}
			
			@Override
			public void mousePressed(MouseEvent arg0) {}
			
			@Override
			public void mouseExited(MouseEvent arg0) {}
			
			@Override
			public void mouseEntered(MouseEvent arg0) {}
			
			@Override
			public void mouseClicked(MouseEvent arg0) {
				if (arg0.getClickCount()==2) {
					a.actionPerformed(null);
				}
			}
		});
	}
	
	public static int getColumnCount() {
		return 3;
	}
	
//	public void addActionListener(ActionListener al) {
//		
//	}
	
	private class DisplayFileTime implements Comparable<DisplayFileTime>{
		private FileTime ft;
		
		public DisplayFileTime(FileTime ft) {
			this.ft=ft;
		}
		
		@Override
		public String toString() {
			return GuiUtil.formatDate(ft.toMillis());
		}

		@Override
		public int compareTo(DisplayFileTime o) {
			return ft.compareTo(o.ft);
		}
	}
	
	public JTable getFileTable() {
		return fileList;
	}
	
}
