/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui.filelist;


import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.io.FileNotFoundException;
import java.util.HashMap;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.RowSorter;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableModel;

import sun.awt.shell.ShellFolder;

public class FileListCellRenderer extends DefaultTableCellRenderer {
	private static final long serialVersionUID = -7021912079797360485L;
	
	private static HashMap<String, Icon> iconCache;
	private FileList fl;
	private static Icon blank;
	
	static {
		iconCache = new HashMap<>();
		blank = new Icon() {
			
			@Override
			public void paintIcon(Component c, Graphics g, int x, int y) {
				
			}
			
			@Override
			public int getIconWidth() {
				return 16;
			}
			
			@Override
			public int getIconHeight() {
				return 16;
			}
		};
		iconCache.put("lnk", blank);
	}
	
	public FileListCellRenderer(FileList fl) {
		this.fl=fl;
	}
	
	@Override
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
		JLabel l= (JLabel) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
		RowSorter<? extends TableModel> rs=table.getRowSorter();
		FileName fn=(FileName) rs.getModel().getValueAt(rs.convertRowIndexToModel(row),0);
//		System.out.println(fn + " "+row);
		String ext=fn.getExt();
		if (ext!=null) {
			if (!ext.equals("exe")){
				Icon i=iconCache.get( ext);
				if (i==null) {
					i=getIcon(fn.toString());
					iconCache.put(ext, i);
				}
				l.setIcon(i);
			} else {
				l.setIcon(getIcon(fn.toString()));
			}
		} else {
			l.setIcon(blank);
		}
		return l;
	}
	
	private Icon getIcon(String f) {

        ShellFolder sf;

        try {        	
        	sf = ShellFolder.getShellFolder(fl.getDir().resolve(f).toFile());        		
        } catch (FileNotFoundException e) {
            return null;
        }

        Image img = sf.getIcon(false);

        if (img != null) {
            return new ImageIcon(img, sf.getFolderType());
        } else {
            return UIManager.getIcon("FileView.fileIcon");
        }
	}
}
